@@ -1,6 +1,6 @@
 #!/usr/bin/python
 import re, locale
-from defaults import lang as defaults
+from defaults.defaults import lang as defaults
 from gettext import gettext as _
 from gettext import ngettext
 from gdebug import *
@@ -8,8 +8,36 @@ from gdebug import *
 FRACTIONS_ALL = 1
 FRACTIONS_NORMAL = 0
 FRACTIONS_ASCII = -1
+FRACTIONS_OFF = -2
 
-class converter:
+USE_FRACTIONS = FRACTIONS_NORMAL
+
+class PossiblyCaseInsensitiveDictionary (dict):
+
+    transformations = ["lower","title","upper"]
+
+    def has_key (self, k):
+        if dict.has_key(self,k): return True
+        else:
+            for t in self.transformations:
+                if hasattr(k,t):
+                    if dict.has_key(self,getattr(k,t)()): return True
+        return False
+
+    def __getitem__ (self, k):
+        if dict.has_key(self,k):
+            return dict.__getitem__(self,k)
+        else:
+            for t in self.transformations:
+                if hasattr(k,t):
+                    nk = getattr(k,t)()
+                    if dict.has_key(self,nk):
+                        return dict.__getitem__(self,nk)
+        # Raise plain old error
+        dict.__getitem__(self,k)
+        
+
+class Converter:
 
     unit_to_seconds = {
     'seconds':1,
@@ -30,11 +58,11 @@ class converter:
     # all possible forms in all languages.
     #
     # See http://www.sbin.org/doc/glibc/libc_8.html
-    time_units = [('seconds',[ngettext('second','seconds',n) for n in range(5)]),
+    time_units = [('seconds',[ngettext('second','seconds',n) for n in range(5)] + ['s.', 'sec', 'secs' ]),
                   # min. = abbreviation for minutes
-                  ('minutes',[_('min.')] + [ngettext('minute','minutes',n) for n in range(5)]),
+                  ('minutes',[_('min'),'min.','min','mins'] + [ngettext('minute','minutes',n) for n in range(5)]),
                   # hrs = abbreviation for hours
-                  ('hours',[_('hrs.')] + [ngettext('hour','hours',n) for n in range(5)]),
+                  ('hours',[_('hrs.'),'hrs','hr'] + [ngettext('hour','hours',n) for n in range(5)]),
                   ('days',[ngettext('day','days',n) for n in range(5)]),
                   ('years',[ngettext('year','years',n) for n in range(5)]),
                   #('decades',[ngettext('decade','decades',n) for n in range(5)]),
@@ -55,7 +83,6 @@ class converter:
         self.add_time_table()
         self.build_converter_dictionary()
         self.build_converter_dictionary(self.v2m_table,density=True)
-        
 
     def add_time_table (self):
         for u,conv in self.unit_to_seconds.items():
@@ -84,7 +111,7 @@ class converter:
                 if not a.upper() in lst: lst.append(a.upper())
                 if not a.lower() in lst: lst.append(a.lower())
             self.units.append((u,lst))
-        self.unit_dict={}
+        self.unit_dict=PossiblyCaseInsensitiveDictionary()
         for itm in self.units:
             key = itm[0]
             variations = itm[1]
@@ -379,7 +406,9 @@ class converter:
             n1 = 0
             for u2,rng in units:
                 conv = self.converter(u,u2)
-                if not conv: return amt,unit
+                if not conv:
+                    print "Warning: no conversion for",u,u2
+                    continue
                 new_amt = conv * amt
                 readability = self.readability_score(new_amt,u2)
                 debug('%s %s, Readability = %s'%(new_amt,u2,readability),6)
@@ -455,20 +484,32 @@ class converter:
         We assume numbers come before time units - surely this will
         break some languages(?). We'll build the parameter in when the
         time comes...
+
+        Return 0 if timestring is unparsable
         """
+        # Before we do our real work, parse times that look like this 0:30
+        # Note the following will be true
+        # 1:30 = 1 1/2 hours
+        # 00:00:20 = 20 seconds
+        if re.match('^\d\d?:\d\d(:\d\d)?$',timestring):
+            times = [locale.atof(s) for s in timestring.split(':')]
+            if len(times) == 3:
+                h,m,s = times
+            else:
+                h,m = times; s = 0
+            return h*60*60 + m*60 + s
         numbers = []
         for match in NUMBER_FINDER.finditer(timestring):
             if numbers: numbers[-1].append(match.start())
             numbers.append([match.start(),match.end()])
         if numbers: numbers[-1].append(None)
-        print numbers
         secs = 0
         for num_start,num_end,section_end in numbers:
             num = frac_to_float(timestring[num_start:num_end])
             unit = timestring[num_end:section_end].strip()
             if self.unit_dict.has_key(unit):
                 conv = self.converter(unit,'seconds')
-                if conv:
+                if conv and num:
                     secs += num * conv
         return secs
 
@@ -509,7 +550,7 @@ time_formatters = {
 def seconds_to_timestring (time, round_at=None, fractions=FRACTIONS_NORMAL):
     time = int(time)
     time_strings = []
-    units = converter.unit_to_seconds.items()
+    units = Converter.unit_to_seconds.items()
     units.sort(lambda a,b: a[1]<b[1] and 1 or a[1]>b[1] and -1 or 0)
     for unit,divisor in units:  
         time_covered = time / int(divisor)
@@ -702,13 +743,31 @@ NUMBER_FINDER = re.compile(NUMBER_FINDER_REGEXP,re.UNICODE)
 RANGE_REGEXP = '([ -]*%s[ -]*|\s*-\s*)'%_('to') # for 'to' used in a range, as in 3-4
 RANGE_MATCHER = re.compile(RANGE_REGEXP[1:-1]) # no parens for this one
 
+
+# We need a special matcher to match known units when they are more
+# than one word. The assumption remains that units should be one word
+# -- but if we already know about two word units, then we should
+# recognize them.
+
+multi_word_units = []
+for canonical_name,other_names in defaults.UNITS:
+    if ' ' in canonical_name: multi_word_units.append(canonical_name)
+    for n in other_names:
+        if ' ' in n: multi_word_units.append(n)
+MULTI_WORD_UNIT_REGEXP = '(' + \
+                       '|'.join([re.escape(unicode(u)) for u in multi_word_units]) \
+                       + ')'
+
+
 # generic ingredient matcher. This is far from a good matcher -- it's
 # used as a fallback to test for things that obviously look like
 # ingredients (e.g. 1 cup milk) that get misparsed by other ingredient
 # parsers. This is often necessary because formats like mealmaster and
 # mastercook are rarely actually followed.
 NUMBER_FINDER_REGEXP2 = NUMBER_FINDER_REGEXP.replace('int','int2').replace('frac','frac2')
-ING_MATCHER_REGEXP = """
+
+try:
+    ING_MATCHER_REGEXP = """
  \s* # opening whitespace
  (?P<amount>
  %(NUMBER_FINDER_REGEXP)s # a number
@@ -718,9 +777,20 @@ ING_MATCHER_REGEXP = """
  %(NUMBER_FINDER_REGEXP2)s)? # and more numbers
  )? # and of course no number is possible
  \s* # Whitespace between number and unit
- (?P<unit>\s*[\w.]+\s+)? # a unit
+ (?P<unit>\s*(%(MULTI_WORD_UNIT_REGEXP)s|[\w.]+))?\s+ # a unit
  (?P<item>.*?)$ # and the rest of our stuff...
- """%locals()
+ """
+    ING_MATCHER_REGEXP = ING_MATCHER_REGEXP%locals()
+except:
+    print 'Failure with local vars...'
+    for s in ['NUMBER_FINDER_REGEXP',
+              'NUMBER_FINDER_REGEXP2',
+              'RANGE_REGEXP',
+              'MULTI_WORD_UNIT_REGEXP',]:
+        try: print 'DOUBLE CHECK',s,'%%(%s)s'%s%locals()
+        except:
+            print 'Failed with ',s,locals()[s]
+    raise
 
 ING_MATCHER = re.compile(ING_MATCHER_REGEXP,
                          re.VERBOSE|re.UNICODE)
@@ -768,31 +838,38 @@ def float_to_frac (n, d=[2,3,4,5,6,8,10,16],approx=0.01,fractions=FRACTIONS_NORM
     """Take a number -- or anything that can become a float --
     and attempt to return a fraction with a denominator in the list `d'. We
     approximate fractions to within approx. i.e. if approx=0.01, then 0.331=1/3"""
-    if not n: return ""
-    n=float(n)
-    i = int(n)
-    if i >= 1:
-        i="%s"%int(n)
+    if USE_FRACTIONS == FRACTIONS_OFF:
+        return float_to_metric(n)
     else:
-        i=""
-    rem = n - int(n)
-    if rem==0:
-        if i:
-            return "%s"%i
+        if not n: return ""
+        n=float(n)
+        i = int(n)
+        if i >= 1:
+            i="%s"%int(n)
         else:
-            return "0"
+            i=""
+        rem = n - int(n)
+        if rem==0:
+            if i:
+                return "%s"%i
+            else:
+                return "0"
+        else:
+            flag = False
+            for div in d:
+                f = fractify(rem,div,approx=approx,fractions=fractions)
+                if f:
+                    return " ".join([i,f]).strip()
+             # use locale-specific metric formatting if fractions don't work
+            return float_to_metric(n)
+
+def float_to_metric(n):
+    """Returns a formatted string in metric format, using locale-specific formatting"""    
+    if len("%s"%n) > 5:
+        return locale.format("%.2f",n,True) # format(formatstring, number, use_thousands_separator)
     else:
-        flag = False
-        for div in d:
-            f = fractify(rem,div,approx=approx,fractions=fractions)
-            if f:
-                return " ".join([i,f]).strip()
-        if len("%s"%n) > 5:
-            # use locale-specific formatting if fractions don't work
-            return locale.format("%.2f",n,True) # format(formatstring, number, use_thousands_separator)
-        else:
-            return locale.format("%s",n,True)
-        
+        return locale.format("%s",n,True)
+    
 def float_string (s):
     """Convert string to a float, assuming it is some sort of decimal number
 
@@ -835,6 +912,9 @@ def float_string (s):
 
 def frac_to_float (s):
     """We assume fractions look like this (I )?N/D"""
+    if NUMBER_WORDS.has_key(s): return NUMBER_WORDS[s]
+    if hasattr(s,'lower') and NUMBER_WORDS.has_key(s.lower()):
+        return NUMBER_WORDS[s.lower()]    
     s = unicode(s)
     m=FRACTION_MATCHER.match(s)
     if m:
@@ -863,12 +943,17 @@ def frac_to_float (s):
         except:
             None
 
+#class ConverterSingleton:
+#    
+#    __impl == Converter
+#
+#    def __init__ (
 
 
 if __name__ == '__main__' and False:
-    class interactive:
+    class InteractiveConverter:
         def __init__ (self):
-            self.c = converter()
+            self.c = Converter()
             self.options = {'Convert':self.converter,
                             'Add':self.adder,
                             'Adjust':self.adjuster,
@@ -944,4 +1029,4 @@ if __name__ == '__main__' and False:
             sys.exit()
         
     
-    #i=interactive()
+    #i=InteractiveConverter()
