@@ -2,12 +2,15 @@ import re, Image, os.path, os, xml.sax.saxutils, time, shutil, urllib, textwrap,
 from gourmet import gglobals, convert
 from gourmet.gdebug import *
 from gettext import gettext as _
+from gourmet.plugin_loader import Pluggable, pluggable_method
+from gourmet.plugin import BaseExporterPlugin, BaseExporterMultiRecPlugin
+from gourmet.threadManager import SuspendableThread
 
 REC_ATTR_DIC = gglobals.REC_ATTR_DIC
 DEFAULT_ATTR_ORDER = gglobals.DEFAULT_ATTR_ORDER
 DEFAULT_TEXT_ATTR_ORDER = gglobals.DEFAULT_TEXT_ATTR_ORDER
 
-class exporter:
+class exporter (SuspendableThread, Pluggable):
     """A base exporter class.
 
     All Gourmet exporters should subclass this class or one of its
@@ -17,6 +20,8 @@ class exporter:
     """
     DEFAULT_ENCODING = 'utf-8'
 
+    name='exporter'
+
     def __init__ (self, rd, r, out,
                   conv=None,
                   imgcount=1,
@@ -26,7 +31,8 @@ class exporter:
                   do_markup=True,
                   use_ml=False,
                   convert_attnames=True,
-                  fractions=convert.FRACTIONS_ASCII
+                  fractions=convert.FRACTIONS_ASCII,
+
                   ):
         """Instantiate our exporter.
 
@@ -45,7 +51,6 @@ class exporter:
                          attribute name staying consistent for processing, since converting attnames
                          will produce locale specific strings.
         """
-        tt = TimeAction('exporter.__init__()',0)
 	self.attr_order=attr_order
         self.text_attr_order = text_attr_order
         self.out = out
@@ -55,36 +60,35 @@ class exporter:
         self.fractions=fractions
         self.use_ml=use_ml
         self.convert_attnames = convert_attnames
-        if not conv: conv=convert.converter()
+        if not conv: conv=convert.Converter()
         self.conv=conv
         self.imgcount=imgcount
-        self.write_head()
         self.images = []
-        for task in order:
-            t=TimeAction('exporter._write_attrs_()',4)
+        self.order = order
+        Pluggable.__init__(self,[BaseExporterPlugin])
+        SuspendableThread.__init__(self,self.name)
+
+    def do_run (self):
+        self.write_head()
+        for task in self.order:
             if task=='image':
                 if self._grab_attr_(self.r,'image'):
                     self.write_image(self.r.image)
             if task=='attr':
                 self._write_attrs_()
-                t.end()
-                t=TimeAction('exporter._write_text_()',4)
+
             elif task=='text':
                 self._write_text_()
-                t.end()
-                t=TimeAction('exporter._write_ings_()',4)            
             elif task=='ings': self._write_ings_()
-            t.end()
         self.write_foot()
-        tt.end()
+
 
     # Internal methods -- ideally, subclasses should have no reason to
     # override any of these methods.
-        
+    @pluggable_method
     def _write_attrs_ (self):
         self.write_attr_head()
         for a in self.attr_order:
-            gglobals.gt.gtk_update()
             txt=self._grab_attr_(self.r,a)
             debug('_write_attrs_ writing %s=%s'%(a,txt),1)
             if txt and (
@@ -95,11 +99,12 @@ class exporter:
                 if (a=='preptime' or a=='cooktime') and a.find("0 ")==0: pass
                 else:
                     if self.convert_attnames:
-                        self.write_attr(REC_ATTR_DIC[a],txt)
+                        self.write_attr(REC_ATTR_DIC.get(a,a),txt)
                     else:
                         self.write_attr(a,txt)
         self.write_attr_foot()
 
+    @pluggable_method
     def _write_text_ (self):
         #print 'exporter._write_text_',self.text_attr_order,'!'
         for a in self.text_attr_order:
@@ -124,7 +129,7 @@ class exporter:
                         txt=self.handle_markup(s)
                     if not self.use_ml: txt = xml.sax.saxutils.unescape(s)
                     if self.convert_attnames:
-                        out_a = gglobals.TEXT_ATTR_DIC[a]
+                        out_a = gglobals.TEXT_ATTR_DIC.get(a,a)
                     else:
                         out_a = a
                     # Goodness this is an ugly way to pass the
@@ -146,20 +151,19 @@ class exporter:
                 #else: print 'exporter: do_markup=False'
                 if not self.use_ml: txt = xml.sax.saxutils.unescape(txt)
                 if self.convert_attnames:
-                    self.write_text(gglobals.TEXT_ATTR_DIC[a],txt)
+                    self.write_text(gglobals.TEXT_ATTR_DIC.get(a,a),txt)
                 else:
                     self.write_text(a,txt)
 
+    @pluggable_method
     def _write_ings_ (self):
         """Write all of our ingredients.
         """
         ingredients = self.rd.get_ings(self.r)
         if not ingredients:
             return
-        gglobals.gt.gtk_update()
         self.write_inghead()
         for g,ings in self.rd.order_ings(ingredients):
-            gglobals.gt.gtk_update()
             if g:
                 self.write_grouphead(g)            
             for i in ings:
@@ -217,7 +221,10 @@ class exporter:
     def _get_amount_and_unit_ (self, ing):
         return self.rd.get_amount_and_unit(ing,fractions=self.fractions)
 
-    # Below are the images inherited exporters should subclass.
+    # Below are the images inherited exporters should
+    # subclass. Subclasses overriding methods should make these
+    # pluggable so that plugins can fiddle about with things as they
+    # see fit.
 
     def write_image (self, image):
         """Write image based on binary data for an image file (jpeg format)."""
@@ -232,22 +239,27 @@ class exporter:
         """Write any necessary footer material at recipe end."""
         pass
 
+    @pluggable_method
     def write_inghead(self):
         """Write any necessary markup before ingredients."""
         self.out.write("\n---\n%s\n---\n"%_("Ingredients"))
 
+    @pluggable_method
     def write_ingfoot(self):
         """Write any necessary markup after ingredients."""
         pass
 
+    @pluggable_method
     def write_attr_head (self):
         """Write any necessary markup before attributes."""
         pass
-    
+
+    @pluggable_method
     def write_attr_foot (self):
         """Write any necessary markup after attributes."""
         pass
 
+    @pluggable_method
     def write_attr (self, label, text):
         """Write an attribute with label and text.
 
@@ -262,6 +274,7 @@ class exporter:
         """
         self.out.write("%s: %s\n"%(label, text.strip()))
 
+    @pluggable_method
     def write_text (self, label, text):
         """Write a text chunk.
 
@@ -322,15 +335,18 @@ class exporter:
         """Make chunk underlined, or the equivalent of"""
         return "_" + chunk + "_"
 
+    @pluggable_method
     def write_grouphead (self, text):
         """The start of group of ingredients named TEXT"""
         self.out.write("\n%s:\n"%text.strip())
 
+    @pluggable_method
     def write_groupfoot (self):
         """Mark the end of a group of ingredients.
         """
         pass
     
+    @pluggable_method
     def write_ingref (self, amount=1, unit=None,
                       item=None, optional=False,
                       refid=None):
@@ -340,6 +356,7 @@ class exporter:
                        unit=unit, item=item,
                        key=None, optional=optional)
 
+    @pluggable_method
     def write_ing (self, amount=1, unit=None, item=None, key=None, optional=False):
         """Write ingredient."""
         if amount:
@@ -387,6 +404,7 @@ class exporter_mult (exporter):
                           fractions=fractions,
                           )
 
+    @pluggable_method
     def write_attr (self, label, text):
         #attr = gglobals.NAME_TO_ATTR[label]
         self.out.write("%s: %s\n"%(label, text))
@@ -419,6 +437,7 @@ class exporter_mult (exporter):
             return self.rd.get_amount_and_unit(ing,mult=self.mult,conv=self.conv,
                                                fractions=self.fractions)
 
+    @pluggable_method
     def write_ing (self, amount=1, unit=None, item=None, key=None, optional=False):
         if amount:
             self.out.write("%s"%amount)
@@ -430,9 +449,11 @@ class exporter_mult (exporter):
             self.out.write(" (%s)"%_("optional"))
         self.out.write("\n")        
 
-class ExporterMultirec:
+class ExporterMultirec (SuspendableThread, Pluggable):
 
-    def __init__ (self, rd, rview, out, one_file=True,
+    name = 'Exporter'
+
+    def __init__ (self, rd, recipe_table, out, one_file=True,
                   ext='txt',
                   conv=None,
                   imgcount=1,
@@ -440,31 +461,19 @@ class ExporterMultirec:
                   exporter=exporter,
                   exporter_kwargs={},
                   padding=None):
-        """Output all recipes in rview into a document or multiple
+        """Output all recipes in recipe_table into a document or multiple
         documents. if one_file, then everything is in one
         file. Otherwise, we treat 'out' as a directory and put
-        individual recipe files within it."""        
+        individual recipe files within it."""
         self.timer=TimeAction('exporterMultirec.__init__()')
         self.rd = rd
-        self.rview = rview
+        self.recipe_table = recipe_table
         self.out = out
         self.padding=padding
-        if not one_file:
-            self.outdir=out
-            if os.path.exists(self.outdir):
-                if not os.path.isdir(self.outdir):
-                    self.outdir=self.unique_name(self.outdir)
-                    os.makedirs(self.outdir)
-            else: os.makedirs(self.outdir)
-        if one_file and type(out)==str:
-            self.ofi=open(out,'wb')
-        else: self.ofi = out
-        self.write_header()
-        self.rcount = 0
-        self.rlen = len(self.rview)
-        self.suspended = False
-        self.terminated = False
-        self.pf = progress_func
+        self.one_file = one_file
+        Pluggable.__init__(self,[BaseExporterMultiRecPlugin])
+        SuspendableThread.__init__(self,self.name)
+        if progress_func: print 'Argument progress_func is obsolete and will be ignored:',progress_func
         self.ext = ext
         self.exporter = exporter
         self.exporter_kwargs = exporter_kwargs
@@ -472,7 +481,6 @@ class ExporterMultirec:
                                                   convert.FRACTIONS_ASCII)
         self.DEFAULT_ENCODING = self.exporter.DEFAULT_ENCODING
         self.one_file = one_file
-        self.rd = rd
 
     def _grab_attr_ (self, obj, attr):
         if attr=='category':
@@ -501,20 +509,37 @@ class ExporterMultirec:
                     raise
             return ret
         
-    def run (self):
+    @pluggable_method
+    def do_run (self):
+        self.rcount = 0
+        self.rlen = len(self.recipe_table)        
+        if not self.one_file:
+            self.outdir=self.out
+            if os.path.exists(self.outdir):
+                if not os.path.isdir(self.outdir):
+                    self.outdir=self.unique_name(self.outdir)
+                    os.makedirs(self.outdir)
+            else: os.makedirs(self.outdir)
+        if self.one_file and type(self.out)==str:
+            self.ofi=open(self.out,'wb')
+        else: self.ofi = self.out
+        self.write_header()
+        self.suspended = False
+        self.terminated = False
         first = True
-        for r in self.rview:
+        for r in self.recipe_table:
             self.check_for_sleep()
-            if self.pf:
-                msg = _("Exported %(number)s of %(total)s recipes")%{'number':self.rcount,'total':self.rlen}
-                self.pf(float(self.rcount)/float(self.rlen), msg)
+            msg = _("Exported %(number)s of %(total)s recipes")%{'number':self.rcount,'total':self.rlen}
+            self.emit('progress',float(self.rcount)/float(self.rlen), msg)
             fn=None
             if not self.one_file:
-                fn=self.generate_filename(r,self.ext)
+                fn=self.generate_filename(r,self.ext,add_id=True)
                 self.ofi=open(fn,'wb')
             if self.padding and not first:
                 self.ofi.write(self.padding)
             e=self.exporter(out=self.ofi, r=r, rd=self.rd, **self.exporter_kwargs)
+            self.connect_subthread(e)
+            e.do_run()
             self.recipe_hook(r,fn,e)
             if not self.one_file:
                 self.ofi.close()
@@ -524,16 +549,18 @@ class ExporterMultirec:
         if self.one_file:
             self.ofi.close()
         self.timer.end()
-        if self.pf: self.pf(1,_("Export complete."))
+        self.emit('progress',1,_("Export complete."))
         print_timer_info()
 
+    @pluggable_method
     def write_header (self):
         pass
 
+    @pluggable_method
     def write_footer (self):
         pass
 
-    def generate_filename (self, rec, ext):
+    def generate_filename (self, rec, ext, add_id=False):
         title=rec.title
         # get rid of potentially confusing characters in the filename
         # Windows doesn't like a number of special characters, so for
@@ -545,7 +572,7 @@ class ExporterMultirec:
                 ntitle += c
         title = ntitle
         # truncate long filenames
-        max_filename_length = 255
+        max_filename_length = 252
         if len(title) > (max_filename_length - 4):
             title = title[0:max_filename_length-4]
         # make sure there is a title
@@ -553,6 +580,9 @@ class ExporterMultirec:
             title = _("Recipe")
         title=title.replace("/"," ")
         title=title.replace("\\"," ")
+        # Add ID #
+        if add_id:
+            title = title + str(rec.id)
         file_w_ext="%s%s%s"%(self.unique_name(title),os.path.extsep,ext)
         return os.path.join(self.outdir,file_w_ext)
 
@@ -576,11 +606,9 @@ class ExporterMultirec:
             return filename
 
     def check_for_sleep (self):
-        gglobals.gt.gtk_update()
         if self.terminated:
             raise "Exporter Terminated!"
         while self.suspended:
-            gglobals.gt.gtk_update()
             if self.terminated:
                 debug('Thread Terminated!',0)
                 raise "Exporter Terminated!"
