@@ -46,6 +46,20 @@ def get_url (url, progress):
 
 class MyBeautifulSoup (BeautifulSoup.ICantBelieveItsBeautifulSoup):
 
+    def __init__ (self, *args, **kwargs):
+        # Avoid invalid doctype decls of the type
+        # <!DOCTYPE foo ... />
+        # From the overly XML zealous folks at sfgate...
+        # http://sfgate.com/cgi-bin/article.cgi?f=/chronicle/archive/2006/08/16/FDG1LKHOMG1.DTL
+        self.PARSER_MASSAGE.append(
+            (re.compile('<!([^<>]*)/>',),
+             lambda x: '<!'+x.group(1)+'>'
+             )
+            )
+        kwargs['avoidParserProblems']=True
+        BeautifulSoup.ICantBelieveItsBeautifulSoup.__init__(self,*args,**kwargs)
+
+    
     def handle_comment (self, text): pass
     def handle_decl (self, data): pass
     def handle_pi (self, text): pass
@@ -110,6 +124,8 @@ class BeautifulSoupScraper:
             post_processing=None
         elif len(rule)==4:
             store_as,tagpath,retmethod,post_processing=rule
+        else:
+            raise "Rule %s is invalid (it should be 3 or 4 items long)."%rule
         tag = self.get_tag_according_to_path(tagpath)
         self.store_tag(store_as,tag,retmethod,post_processing)
 
@@ -180,10 +196,10 @@ class BeautifulSoupScraper:
                 else: get_to += 1
             if get_to:
                 ret = base.fetch(step.get('tag'),
-                                 step.get('attributes'),
+                                 step.get('attributes',{}),
                                  get_to)
             else:
-                ret = base.fetch(step.get('tag'),step.get('attributes'))
+                ret = base.fetch(step.get('tag'),step.get('attributes',{}))
         if ret:
             # if we have moveto, we do it with our index -- for
             # example, if we have step['moveto']='parent', we grab the
@@ -245,7 +261,11 @@ class GenericScraper (BeautifulSoupScraper):
     def __init__ (self):
         BeautifulSoupScraper.__init__(self,
             [['text',
-              [{'tag':'body'}],
+              [{'tag':'body',
+                # Believe it or not, I've found recipe webpages with
+                # more than one body tag
+                'index':[0,None],
+                }],
               'text',
               ],
              ['images',
@@ -329,11 +349,7 @@ img_src_regexp = re.compile('<img[^>]+src=[\'\"]([^\'"]+)')
 
 def get_image_from_tag (iurl, page_url):
     if not iurl: return
-    if iurl[0]=="/":
-        base_url = "/".join(page_url.split('/')[0:3]) + '/'
-    else:
-        base_url = "/".join(page_url.split('/')[0:-1]) + '/'
-    iurl = base_url + iurl
+    iurl = urllib.basejoin(page_url,iurl)
     tmpfi,info=urllib.urlretrieve(iurl)
     ifi=file(tmpfi,'rb')
     retval=ifi.read()
@@ -431,7 +447,13 @@ class WebPageImporter (importer.importer):
         our web page.
         """
         debug('Scraping url %s'%self.url,0)
-        self.d = scrape_url(self.url, progress=self.prog)
+        try:
+            self.d = scrape_url(self.url, progress=self.prog)
+        except:
+            print 'Trouble using default recipe filter to download %s'%self.url
+            traceback.print_exc()
+            print 'We will use a generic importer instead.'
+            self.d = {}
         debug('Scraping url returned %s'%self.d,0)
         do_generic = not self.d
         if not do_generic:
@@ -464,9 +486,12 @@ class WebPageImporter (importer.importer):
             self.prog(-1,_("Don't recognize this webpage. Using generic importer..."))
             gs = GenericScraper()
             text,images = gs.scrape_url(self.url, progress=self.prog)
+            if not text and not images:
+                raise "Unable to obtain text or images from url %s"%self.url
             import interactive_importer
             ii = interactive_importer.InteractiveImporter(self.rd)
             ii.set_text(text)
+            ii.add_attribute('link',self.url)
             ii.set_images(images)
             ii.run()
             if self.prog: self.prog(1,_('Import complete.'))
@@ -476,6 +501,8 @@ class WebPageImporter (importer.importer):
         """Get URL based on template stored in d
         """
         self.start_rec()
+        # Set link
+        self.rec['link']=self.url
         # Add webpage as source
         if self.add_webpage_source:
             # add Domain as source
@@ -486,31 +513,36 @@ class WebPageImporter (importer.importer):
             elif src: src = [src,add_str]
             else: src = domain # no parens if we're the only source
             self.d['source']=src
-            # and add a note to instructions with the full URL
-            instr=self.d.get('instructions',[])
-            add_str=_('Retrieved from %(url)s.')%{'url':self.url}
-            if type(instr)==list: instr.append(add_str)
-            else: instr = [instr,add_str]
-            self.d['instructions']=instr
         for k,v in self.d.items():
             debug('processing %s:%s'%(k,v),1)
             if self.prog: self.prog(-1,_('Importing recipe'))
+            # parsed ingredients...
             if k=='ingredient_parsed':
                 if type(v) != list: v=[v]
                 for ingdic in v:
+                    
                     if self.prog: self.prog(-1,_('Processing ingredients'))
                     # we take a special keyword, "text", which gets
                     # parsed
                     if ingdic.has_key('text'):
                         d = self.rd.ingredient_parser(ingdic['text'],conv=self.conv)
                         if d:
-                            for dk,dv in d.items(): ingdic[dk]=dv
+                            for dk,dv in d.items():
+                                if not ingdic.has_key(dk) or not ingdic[dk]:
+                                    ingdic[dk]=dv
+                        elif not ingdic.has_key('item'):
+                            ingdic['item']=ingdic['text']
                         del ingdic['text']
                     self.start_ing(**ingdic)
                     self.commit_ing()
+                continue
+
+            # Listy stuff...
             elif type(v)==list:
                 if k in self.JOIN_AS_PARAGRAPHS: v = "\n".join(v)
                 else: v = " ".join(v)
+
+            # Ingredients in blocks
             if k == 'ingredient_block':
                 for l in v.split('\n'):
                     if self.prog: self.prog(-1,_('Processing ingredients.'))
@@ -518,15 +550,18 @@ class WebPageImporter (importer.importer):
                     if dic:
                         self.start_ing(**dic)
                         self.commit_ing()
+                        
             elif k == 'image':
                 try:
-                    if v: self.rec['image']=get_image_from_tag(v,self.url)
+                    if v: img = get_image_from_tag(v,self.url)
                 except:
-                    debug('Error retrieving image',0)
-                    debug('tried to retrieve image from %s'%v,0)
-                    raise
+                    print 'Error retrieving image'
+                    print 'tried to retrieve image from %s'%v
+                else:
+                    if img:
+                        self.rec['image'] = img
             else: self.rec[k]=v
+        #print 'COMMITTING RECIPE',self.rec
         self.commit_rec()
         if self.prog: self.prog(1,_('Import complete.'))
 
-    
