@@ -2,26 +2,27 @@
 import os,stat,re,time,StringIO
 from gourmet import keymanager, convert, ImageExtras
 from gourmet.gdebug import debug, TimeAction, print_timer_info
-from gourmet.gglobals import gt, use_threads
+import gourmet.gglobals
 import xml.sax.saxutils
 from gettext import gettext as _
-import gourmet.dialog_extras as de
+import gourmet.gtk_extras.dialog_extras as de
 import re
+from gourmet.threadManager import SuspendableThread
 
-class importer:
+class importer (SuspendableThread):
 
     """Base class for all importers. We provide an interface to the recipe database
     for importers to use. Basically, the importer builds up a dictionary of properties inside of
     self.rec and then commits that dictionary with commit_rec(). Similarly, ingredients are built
     as self.ing and then committed with commit_ing()."""
     
-    def __init__ (self, rd, threaded=False, total=0, prog=None, do_markup=True,conv=None,rating_converter=None):
+    def __init__ (self, rd, total=0, prog=None, do_markup=True,conv=None,rating_converter=None,
+                  name='importer'):
         """rd is our recipeData instance. Total is used to keep track of progress
         with function progress. do_markup should be True if instructions and modifications
         come to us unmarked up (i.e. if we need to escape < and &, etc."""
-
         timeaction = TimeAction('importer.__init__',10)
-        if not conv: self.conv = convert.converter()
+        if not conv: self.conv = convert.Converter()
         self.id_converter = {} # a dictionary for tracking named IDs
         self.total = total
         self.prog = prog
@@ -36,7 +37,6 @@ class importer:
         #self.rd.add_hooks = []
         self.position=0
         self.group=None
-        self.threaded=threaded
         # allow threaded calls to pause
         self.suspended = False
         # allow threaded calls to be terminated (this
@@ -58,15 +58,12 @@ class importer:
         else:
             debug('Making new keymanager',2)
             self.km=keymanager.KeyManager(rm=self.rd)
-        if not self.threaded:
-            ## run ourselves, unless we're threaded, in which
-            ## case we'll want to call run explicitly from outside.
-            self.run()
-            self._run_cleanup_()
         timeaction.end()
+        SuspendableThread.__init__(self,
+                                   name=name)
     # end __init__
 
-    def run (self):
+    def do_run (self):
         debug('Running ing hooks',0)
         for i in self.added_ings:
             for h in self.rd_orig_ing_hooks:
@@ -87,16 +84,13 @@ class importer:
 
     def check_for_sleep (self):
         timeaction = TimeAction('importer.check_for_sleep',10)
-        gt.gtk_update()
+        #gt.gtk_update()
         if self.terminated:
             raise gt.Terminated("Importer Terminated!")
         while self.suspended:
-            gt.gtk_update()
+            #gt.gtk_update()
             if self.terminated:
                 raise gt.Terminated("Importer Terminated!")
-            if not use_threads:
-                # we only sleep a little while if we're suspended!
-                time.sleep(0.1)
             else:
                 time.sleep(1)
         timeaction.end()
@@ -131,10 +125,13 @@ class importer:
 
         This is frequently a fallback for bad input -- we try to make
         sure we don't lose data this way."""
+        # We add e.g. "Servings: lots" to instructions -- REC_ATTR_DIC
+        # gets us the properly i18n'd name
+        text_to_add = gourmet.gglobals.REC_ATTR_DIC[attr]+': '+recdic[attr]
         if not recdic.has_key('instructions'):
-            recdic['instructions']=recdic[attr]
+            recdic['instructions']=text_to_add
         else:
-            recdic['instructions']=recdic['instructions']+'\n'+recdic[attr]
+            recdic['instructions']=recdic['instructions']+'\n'+text_to_add
         del recdic[attr]
 
     def commit_rec (self):
@@ -146,15 +143,16 @@ class importer:
         # instructions.
         if self.rec.has_key('servings'):
             servs=self.convert_servings(self.rec['servings'])
-            if servs:
+            if servs != None:
                 self.rec['servings']=str(servs)
             else:
                 self._move_to_instructions(self.rec,'servings')
         # Check preptime and cooktime
         for t in ['preptime','cooktime']:
             if self.rec.has_key(t) and type(self.rec[t])!=int:
+                print 'IMPORTING TEXT TIME %s: "%s"'%(t,self.rec[t])
                 secs = self.conv.timestring_to_seconds(self.rec[t])
-                if secs:
+                if secs != None:
                     self.rec[t]=secs
                 else:
                     self._move_to_instructions(self.rec,t)
@@ -177,11 +175,22 @@ class importer:
         debug('commiting recipe %s'%self.rec,0)
         # Check for images without thumbnails
         if self.rec.get('image',None) and not self.rec.get('thumb',None):
-            print 'Adding thumbnail.'
-            img = ImageExtras.get_image_from_string(self.rec['image'])
-            thumb = ImageExtras.resize_image(img,40,40)
-            self.rec['thumb'] = ImageExtras.get_string_from_image(thumb)
-        r = self.rd.add_rec(self.rec)
+            if not self.rec['image']: del self.rec['image']
+            else:
+                img = ImageExtras.get_image_from_string(self.rec['image'])
+                if img:
+                    thumb = ImageExtras.resize_image(img,40,40)
+                    self.rec['thumb'] = ImageExtras.get_string_from_image(thumb)
+                    # Make sure our image is properly formatted...
+                    self.rec['image'] = ImageExtras.get_string_from_image(img)
+                else:
+                    print "ODD: we got no image from ",self.rec['image'][:100]
+                    print 'Deleting "image"'
+                    del self.rec['image']
+                    del self.rec['thumb']                    
+        r = self.rd.add_rec(self.rec, accept_ids=True) # See doc on add_rec 
+        # Update hash-keys...
+        self.rd.update_hashes(r)
         tt.end()
         self.added_recs.append(r)
         if remembered_rating: self.rating_converter.add(r.id,remembered_rating)
@@ -214,10 +223,15 @@ class importer:
             
     def start_ing (self, **kwargs):
         timeaction = TimeAction('importer.start_ing',10)
-        gt.gtk_update()
+        #gt.gtk_update()
         self.ing=kwargs
-        self.ing['id']=self.rec['id']
-        debug('ing ID %s, recipe ID %s'%(self.ing['id'],self.rec['id']),0)
+        if self.ing.has_key('id'):
+            self.ing['recipe_id']=self.ing['id']
+            del self.ing['id']
+            print 'WARNING: setting ingredients ID is deprecated. Assuming you mean to set recipe_id'
+        elif self.rec.has_key('id'):
+            self.ing['recipe_id']=self.rec['id']
+        debug('ing ID %s, recipe ID %s'%(self.ing['recipe_id'],self.rec['id']),0)
         timeaction.end()
                  
     def commit_ing (self):
@@ -272,7 +286,7 @@ class importer:
         """We should NEVER get non-numeric amounts.
         Amounts must contain [/.0-9 ] e.g. 1.2 or 1 1/5
         or 1/3 etc."""
-        gt.gtk_update()
+        #gt.gtk_update()
         self.ing['amount'],self.ing['rangeamount']=parse_range(amount)
         timeaction.end()
 
@@ -330,14 +344,6 @@ def string_to_rating (s):
         bottom = float(convert.frac_to_float(m.groups()[2]))
         return int(top/bottom * 10)
 
-            
-        
-        
-        
-        
-    
-
-
 class MultipleImporter:
     def __init__ (self,grm,imports):
         """GRM is a GourmetRecipeManager instance.
@@ -349,6 +355,7 @@ class MultipleImporter:
         then run with c.run()
         """
         self.imports = imports
+        self.added_recs = []
         self.grm = grm
         self.total_size = 0
         self.current_prog = 0
@@ -363,7 +370,7 @@ class MultipleImporter:
                 self.total_size += size
                 self.sizes[fn]=size
 
-    def run (self):
+    def do_run (self):
         for importer,fn in self.imports:
             ic,args,kwargs = importer
             self.fn = fn
@@ -383,6 +390,10 @@ class MultipleImporter:
             self.resume = self.iclass.resume
             self.iclass.run()
             self.current_prog += self.current_percentage
+            if hasattr(self.iclass,'added_recs'):
+                self.added_recs += self.iclass.added_recs
+            else:
+                print 'ODD:',self.iclass,'has no added_recs'
         self.grm.set_progress_thr(1,'Import complete!')
     
     def show_progress (self, prog, msg):
@@ -466,21 +477,20 @@ class RatingConverter:
     user convert those to stars as they see fit.
     """
     
-    conversions = {'excellent':5,
-                   'great':4,
-                   'good':3,
-                   'fair':2,
-                   'okay':2,
-                   'poor':1,
-                   _('Excellent').lower():5,
-                   _('Great').lower():4,
-                   _('Good').lower():3,
-                   _('Fair').lower():2,
-                   _('Poor').lower():1,
-                   _('Okay').lower():2,
+    conversions = {'excellent':10,
+                   'great':8,
+                   'good':6,
+                   'fair':4,
+                   'okay':4,
+                   'poor':2,
+                   _('excellent').lower():10,
+                   _('great').lower():8,
+                   _('good').lower():6,
+                   _('fair').lower():4,
+                   _('okay').lower():4,
+                   _('poor').lower():2,
                    }
     
-
     def __init__ (self):
         self.to_convert = {}
         self.got_conversions = False
@@ -503,17 +513,21 @@ class RatingConverter:
         for v in self.to_convert.values():
             if not need_conversions and not self.conversions.has_key(v.lower()):
                 need_conversions = True
-            if not v in ratings: ratings.append(v)
+            if v not in ratings:
+                ratings.append(v)
+                conv = string_to_rating(v)
+                if conv: self.conversions[v] = conv
         if need_conversions:
             self.conversions = de.get_ratings_conversion(ratings,star_generator,defaults=self.conversions)
 
     def do_conversions (self, db):
         if not self.got_conversions:
-            print 'Get conversion!'
             self.get_conversions()
         for id,rating in self.to_convert.items():
             try:
-                db.modify_rec(db.get_rec(id),{'rating':self.conversions[str(rating).lower()]})
+                if not self.conversions.has_key(rating) and hasattr(rating,'lower') and self.conversions.has_key(rating.lower()):
+                    rating = rating.lower()
+                db.modify_rec(db.get_rec(id),{'rating':self.conversions[rating]})
             except:
                 print 'wtf... problem with rating ',rating,'for recipe',id
                 raise
