@@ -64,7 +64,7 @@ class SimpleGladeApp:
             except TypeError:
                 setattr(self, key, value)
         self.glade = None
-        self.install_custom_handler(self.custom_handler)
+        #self.install_custom_handler(self.custom_handler)
         self.glade = self.create_glade(self.glade_path, root, domain)
         if root:
             self.main_widget = self.get_widget(root)
@@ -164,28 +164,28 @@ class SimpleGladeApp:
                         prefix_action = prefix_actions_d[prefix]
                         prefix_action(widget)
 
-    def custom_handler(self,
-            glade, function_name, widget_name,
-            str1, str2, int1, int2):
-        """
-        Generic handler for creating custom widgets, internally used to
-        enable custom widgets (custom widgets of glade).
-
-        The custom widgets have a creation function specified in design time.
-        Those creation functions are always called with str1,str2,int1,int2 as
-        arguments, that are values specified in design time.
-
-        Methods of classes inheriting from SimpleGladeApp are used as
-        creation functions automatically.
-
-        If a custom widget has create_foo as creation function, then the
-        method named create_foo is called with str1,str2,int1,int2 as arguments.
-        """
-        try:
-            handler = getattr(self, function_name)
-            return handler(str1, str2, int1, int2)
-        except AttributeError:
-            return None
+    #def custom_handler(self,
+    #        glade, function_name, widget_name,
+    #        str1, str2, int1, int2):
+    #    """
+    #    Generic handler for creating custom widgets, internally used to
+    #    enable custom widgets (custom widgets of glade).
+    #
+    #    The custom widgets have a creation function specified in design time.
+    #    Those creation functions are always called with str1,str2,int1,int2 as
+    #    arguments, that are values specified in design time.
+    #
+    #    Methods of classes inheriting from SimpleGladeApp are used as
+    #    creation functions automatically.
+    #
+    #    If a custom widget has create_foo as creation function, then the
+    #    method named create_foo is called with str1,str2,int1,int2 as arguments.
+    #    """
+    #    try:
+    #        handler = getattr(self, function_name)
+    #        return handler(str1, str2, int1, int2)
+    #    except AttributeError:
+    #        return None
 
     def gtk_widget_show(self, widget, *args):
         """
@@ -295,8 +295,8 @@ class SimpleGladeApp:
         """
         pass
 
-    def install_custom_handler(self, custom_handler):
-        gtk.glade.set_custom_handler(custom_handler)
+    #def install_custom_handler(self, custom_handler):
+    #    gtk.glade.set_custom_handler(custom_handler)
 
     def create_glade(self, glade_path, root, domain):
         return gtk.glade.XML(self.glade_path, root, domain)
@@ -344,19 +344,25 @@ class ConvenientImporter (importer.importer):
 
     def add_ing_from_text (self, txt):
         self.set_added_to(True)
-        txt=txt.strip()
+        try:
+            txt=unicode(txt.strip())
+        except UnicodeDecodeError:
+            print 'Weird -- ignoring unicode error with ',txt
+            txt = txt.strip()
         txt = re.sub('\s+',' ',txt)
         if not txt: return        
         mm = convert.ING_MATCHER.match(txt)
         if mm:
-            groups = mm.groups()
-            amount = groups[convert.ING_MATCHER_AMT_GROUP]
-            unit = groups[convert.ING_MATCHER_UNIT_GROUP]
-            item = groups[convert.ING_MATCHER_ITEM_GROUP]
-            # If our unit isn't familiar, don't add it as a unit!
-            if unit and not self.conv.unit_dict.has_key(unit.strip()):
-                unit = ''
-                item = unit + item
+            amount = mm.group(convert.ING_MATCHER_AMT_GROUP)
+            unit = mm.group(convert.ING_MATCHER_UNIT_GROUP)
+            item = mm.group(convert.ING_MATCHER_ITEM_GROUP)
+            print 'Parsed ingredient: "%s"'%txt,"into:",amount,unit,item
+            # If our unit isn't familiar and is longer than 2
+            # characters, don't add it as a unit! (this lets most
+            # abbreviations through)
+            if unit and not self.conv.unit_dict.has_key(unit.strip()) and len(unit.strip())>2:
+                item = unit + ' ' + item
+                unit = ''                
         else:
             print 'Unable to parse ingredient from text "%s"'%txt
             print 'Setting amount and unit to None'
@@ -388,9 +394,9 @@ class InteractiveImporter (SimpleGladeApp, ConvenientImporter):
         self.parser_to_choice = {
             'ingredient':'Ingredient',
             'ingredients':'Ingredients',
-            #'instructions':'Instructions',
+            'instructions':'Instructions',
             'None':'Ignore',
-            #'title':'Title',
+            'title':'Title',
             }
         self.added_to = False
         self.attdic = gglobals.REC_ATTR_DIC 
@@ -487,6 +493,9 @@ class InteractiveImporter (SimpleGladeApp, ConvenientImporter):
     
     def set_text (self, txt):
         """Set raw text."""
+        txt = unicode(txt)
+        if txt.strip()=='':
+            raise ValueError("There is no text to set")
         self.textbuffer = gtk.TextBuffer()
         self.textview.set_buffer(self.textbuffer)
         parsed = self.parser.parse(txt,progress=self.progress)
@@ -588,7 +597,10 @@ class InteractiveImporter (SimpleGladeApp, ConvenientImporter):
         if n >= len(self.sections): n = len(self.sections)-1
         elif n < 0: n = 0
         self.curmark = n
-        s,e=self.sections[n]        
+        if len(self.sections)-1 <= n:
+            print "There is no section ",n
+            return 
+        s,e=self.sections[n]
         start_itr=self.textbuffer.get_iter_at_mark(s)
         end_itr = self.textbuffer.get_iter_at_mark(e)
         # Check where our current section is
@@ -681,12 +693,12 @@ class InteractiveImporter (SimpleGladeApp, ConvenientImporter):
             tags = itr.get_tags()
         action = None
         for t in tags:
-            if self.parser_to_choice.has_key(t.props.name):
-                action = self.parser_to_choice[t.props.name]
-            elif self.attdic.has_key(t.props.name):
-                action = self.attdic[t.props.name]
-            elif self.textattdic.has_key(t.props.name):
-                action = self.textattdic[t.props.name]
+            if self.parser_to_choice.has_key(t.get_property('name')):
+                action = self.parser_to_choice[t.get_property('name')]
+            elif self.attdic.has_key(t.get_property('name')):
+                action = self.attdic[t.get_property('name')]
+            elif self.textattdic.has_key(t.get_property('name')):
+                action = self.textattdic[t.get_property('name')]
         self.set_current_action(action)
         
     #-- InteractiveImporter.on_cursor_moved }
@@ -734,12 +746,15 @@ class InteractiveImporter (SimpleGladeApp, ConvenientImporter):
         st,end = self.textbuffer.get_selection_bounds()
         while end.starts_line() and end.backward_char():
             end = self.textbuffer.get_iter_at_offset(end.get_offset()-1)
+
+        # Here's how we would add a "Remove" button -- but this really
+        # requires rethinking the whole process...
+        #
+        #anchor = self.textbuffer.create_child_anchor(end)
+        #button = gtk.Button('_Remove tag'); button.show()
+        #self.textview.add_child_at_anchor(button,anchor)
+        
         self.textbuffer.insert_with_tags(end,']',self.markup_tag)
-        # We'll do something real soon...
-        #self.resultbuffer.insert(
-        #    self.resultbuffer.get_end_iter(),
-        #    "%s: %s\n"%(active_txt,selection)
-        #    )
         action = self.actions[active_txt]
         if self.action_to_label.has_key(active_txt):
             active_txt = self.action_to_label[active_txt]
@@ -761,10 +776,10 @@ class InteractiveTextImporter (InteractiveImporter):
 
 def main():
     from gourmet import recipeManager
-    rd = recipeManager.RecipeManager(**recipeManager.dbargs)
+    rd = recipeManager.RecipeManager(file='/tmp/foo.db')
     window1 = InteractiveImporter(rd)
     window1.set_text(
-        """
+        u"""
 Quick Pesto Dinner
 
 Category: Quick, Easy, Summer
@@ -772,13 +787,8 @@ Yield: 2 Servings
 
 Ingredients:
 1-2 c. fresh basil
-1/4-1/2 c. olive oil
-1/4-1/2 c. pine nuts or walnuts
-1/4 c. parmesan cheese
-A handful of cheese
-Some sausages
-3-6 cloves garlic
-1/2 lb. whole-wheat spaghetti
+\xbc  cup minced fresh ginger (about 6 inches ginger root, peeled)
+\xbc lb. whole-wheat spaghetti
 1-2 fresh tomatoes
 
 To accompany dish:
@@ -795,6 +805,7 @@ Chop up tomatoes roughly.
 Toss spaghetti in pesto and tomatoes.
 """)
     window1.run()        
+    return window1
 
 if __name__ == '__main__':
-    main()
+    ii = main()
