@@ -14,9 +14,9 @@ class Mx2Cleaner:
             self.toss_regexp = self.toss_regexp + r + "|"
         self.toss_regexp = self.toss_regexp[0:-1] + ")"
         self.toss_regexp = re.compile(self.toss_regexp)
-        self.attr_regexp = '(<[^>]+?)\"([^=]+\"[^=]+)\"'
+        self.attr_regexp = '(<[^>]+?)\"([^=]+[\"<>][^=]+)\"'
         self.attr_regexp = re.compile(self.attr_regexp)
-        self.encodings = ['iso8859','ascii','latin_1','cp850','cp1252','utf-8']
+        self.encodings = ['cp1252','iso8859','ascii','latin_1','cp850','utf-8']
         
     def cleanup (self, infile, outfile):
         infile = open(infile,'r')
@@ -210,9 +210,20 @@ class MastercookXMLHandler (xml.sax.ContentHandler, importer.importer):
             self.bufs.remove('dbuf')
             self._add_to_instructions(self.dbuf.strip())
 
-    # these all add to instructions
+    # this also gets added to instructions
     Desc_handler = DirT_handler
-    Note_handler = DirT_handler
+
+    def Note_handler (self, start=False, end=False, attrs=None):
+        if start:
+            self.dbuf = ""
+            self.bufs.append('dbuf')
+        if end:
+            self.bufs.remove('dbuf')
+            buf = xml.sax.saxutils.unescape(self.dbuf.strip())
+            if self.rec.has_key('modifications'):
+                self.rec['modifications'] = self.rec['modifications'] + "\n%s"%buf
+            else:
+                self.rec['modifications'] = buf
 
     def IPrp_handler (self, start=False, end=False, attrs=None):
         if start:
