@@ -1,15 +1,15 @@
 #!/usr/bin/env python
-import gtk.glade, gtk, time, re, gtk.gdk, gobject
-import WidgetSaver, Undo, ratingWidget
+import gtk.glade, gtk, time, re, gtk.gdk, gobject, pango
+from gtk_extras import WidgetSaver, ratingWidget
 from ImageExtras import get_pixbuf_from_jpg
-import dialog_extras as de
-import treeview_extras as te
-import cb_extras as cb
-import convert
+from gtk_extras import dialog_extras as de
+from gtk_extras import treeview_extras as te
+from gtk_extras import cb_extras as cb
+import convert, Undo
 from gglobals import *
 from gdebug import debug
-import mnemonic_manager
-import pageable_store
+from gtk_extras import mnemonic_manager
+from gtk_extras import pageable_store
 from gettext import gettext as _
 from gettext import ngettext
 
@@ -25,6 +25,7 @@ class RecIndex:
     def __init__ (self, glade, rd, rg, editable=False):
         #self.visible = 1 # can equal 1 or 2
         self.editable=editable
+        self.selected = True        
         self.rtcols=rg.rtcols
         self.rtcolsdic=rg.rtcolsdic
         self.rtwidgdic=rg.rtwidgdic
@@ -32,11 +33,8 @@ class RecIndex:
         self.glade = glade
         self.rd = rd
         self.rg = rg
-        self.srchentry=self.glade.get_widget('rlistSearchbox')
-	self.limitButton = self.glade.get_widget('rlAddButton')
-        # Don't # allow for special keybindings
-        #self.srchentry.connect('key_press_event',self.srchentry_keypressCB)
         self.searchByDic = {
+            unicode(_('anywhere')):'anywhere',
             unicode(_('title')):'title',
             unicode(_('ingredient')):'ingredient',
             unicode(_('instructions')):'instructions',
@@ -46,7 +44,8 @@ class RecIndex:
             #_('rating'):'rating',
             unicode(_('source')):'source',
             }
-        self.searchByList = [_('title'),
+        self.searchByList = [_('anywhere'),
+                             _('title'),
                              _('ingredient'),
                              _('category'),
                              _('cuisine'),
@@ -62,9 +61,18 @@ class RecIndex:
         #    "c":_("category"),
         #    "u":_("cuisine"),
         #    's':_("source"),
-        #    }
+        #    }        
+        self.setup_search_actions()
+        self.setup_widgets()
+
+    def setup_widgets (self):
+        self.srchentry=self.glade.get_widget('rlistSearchbox')
+	self.limitButton = self.glade.get_widget('rlAddButton')
+        # Don't # allow for special keybindings
+        #self.srchentry.connect('key_press_event',self.srchentry_keypressCB)        
         self.SEARCH_MENU_KEY = "b"
-        self.srchLimitBar=self.glade.get_widget('srchLimitBar')
+        self.srchLimitBar = self.glade.get_widget('srchLimitBar')
+        assert(self.srchLimitBar)
         self.srchLimitBar.hide()
         self.srchLimitLabel=self.glade.get_widget('srchLimitLabel')
         self.srchLimitClearButton = self.glade.get_widget('srchLimitClear')
@@ -72,13 +80,18 @@ class RecIndex:
         self.srchLimitDefaultText=self.srchLimitText
         self.searchButton = self.glade.get_widget('searchButton')
         self.rSearchByMenu = self.glade.get_widget('rlistSearchByMenu')
-        cb.set_model_from_list(self.rSearchByMenu, self.searchByList)
+        cb.set_model_from_list(self.rSearchByMenu, self.searchByList, expand=False)
         cb.setup_typeahead(self.rSearchByMenu)
         self.rSearchByMenu.set_active(0)
         self.rSearchByMenu.connect('changed',self.search_as_you_type)
         self.sautTog = self.glade.get_widget('searchAsYouTypeToggle')
-        self.sautTog.connect('toggled',self.toggleTypeSearchCB)
+        self.search_actions.get_action('toggleSearchAsYouType').connect_proxy(self.sautTog)
         self.regexpTog = self.glade.get_widget('regexpTog')
+        self.searchOptionsBox = self.glade.get_widget('searchOptionsBox')
+        self.search_actions.get_action('toggleShowSearchOptions').connect_proxy(
+            self.glade.get_widget('searchOptionsToggle')
+            )
+        self.search_actions.get_action('toggleRegexp').connect_proxy(self.regexpTog)
         self.rectree = self.glade.get_widget('recTree')
         self.rectree.connect('start-interactive-search',lambda *args: self.srchentry.grab_focus())
         self.prev_button = self.glade.get_widget('prevButton')
@@ -102,9 +115,11 @@ class RecIndex:
             'categorySearch' : lambda *args: self.set_search_by('category'),
             'cuisineSearch' : lambda *args: self.set_search_by('cuisine'),
             'search' : self.search,
+            'searchBoxActivatedCB':self.search_entry_activate_cb,
             'rlistReset' : self.reset_search,
             'rlistLimit' : self.limit_search,
             'search_as_you_type_toggle' : self.toggleTypeSearchCB,})
+        self.toggleTypeSearchCB(self.sautTog)
         # this has to come after the type toggle is connected!
         self.rg.conf.append(WidgetSaver.WidgetSaver(
             self.sautTog,
@@ -121,7 +136,7 @@ class RecIndex:
         # setup a history
         self.uim=self.glade.get_widget('undo_menu_item')
         self.rim=self.glade.get_widget('redo_menu_item')
-        self.raim=self.glade.get_widget('reapply_menu_item')        
+        self.raim=self.glade.get_widget('reapply_menu_item')
         self.history = Undo.UndoHistoryList(self.uim,self.rim,self.raim)
         # Fix up our mnemonics with some heavenly magic
         self.mm = mnemonic_manager.MnemonicManager()
@@ -130,12 +145,31 @@ class RecIndex:
         self.mm.add_treeview(self.rectree)
         self.mm.fix_conflicts_peacefully()
 
+    def setup_search_actions (self):
+        self.search_actions = gtk.ActionGroup('SearchActions')
+        self.search_actions.add_toggle_actions([
+            ('toggleRegexp',None,_('Use regular expressions in search'),
+             None,_('Use regular expressions (an advanced search language) in text search'),
+             self.toggleRegexpCB),
+            ('toggleSearchAsYouType',None,_('Search as you type'),None,
+             _('Search as you type (turn off if search is too slow).'),
+             self.toggleTypeSearchCB
+             ),
+            ('toggleShowSearchOptions',
+             None,
+             _('Show Search _Options'),
+             None,
+             _('Show advanced searching options'),
+             self.toggleShowSearchOptions),
+            ])
+
     def setup_search_views (self):
         """Setup our views of the database."""
         self.last_search = {}
-        self.rvw = self.rd.fetch_all(self.rd.rview,deleted=False)
+        #self.rvw = self.rd.fetch_all(self.rd.recipe_table,deleted=False)
         self.searches = self.default_searches[0:]
         self.sort_by = []
+        self.rvw = self.rd.search_recipes(self.searches,sort_by=self.sort_by)
 
     def make_rec_visible (self, *args):
         """Make sure recipe REC shows up in our index."""
@@ -146,6 +180,16 @@ class RecIndex:
         #self.visible.append(rec.id)
         #if not self.rg.wait_to_filter:
         #    self.rmodel_filter.refilter()
+
+    def search_entry_activate_cb (self, *args):
+        if self.rmodel._get_length_()==1:
+            self.rec_tree_select_rec()
+        else:
+            if not self.search_as_you_type:
+                self.search()
+                gobject.idle_add(lambda *args: self.limit_search())
+            else:
+                self.limit_search()
     
     def rmodel_page_changed_cb (self, rmodel):
         if rmodel.page==0:
@@ -164,14 +208,13 @@ class RecIndex:
 
     def rmodel_sort_cb (self, rmodel, sorts):
         self.sort_by = sorts
-        print 'Sorting by ',sorts
         self.last_search = {}
         self.search()
         #self.do_search(None,None)
 
     def create_rmodel (self, vw):
         self.rmodel = RecipeModel(vw,self.rd,per_page=self.prefs.get('recipes_per_page',12))
-        self.set_reccount()
+        #self.set_reccount() # This will be called by the rmodel_page_changed_cb
     
     def setup_rectree (self):
         """Create our recipe treemodel."""
@@ -188,12 +231,13 @@ class RecIndex:
         self.setup_reccolumns()
         # this has to come after columns are added or else adding columns resets out column order!
         self.rectree_conf=te.TreeViewConf(self.rectree,
-                                          hidden=self.prefs.get('rectree_hidden_columns',[]),
+                                          hidden=self.prefs.get('rectree_hidden_columns',DEFAULT_HIDDEN_COLUMNS),
                                           order=self.prefs.get('rectree_column_order',{}))
         self.rectree_conf.apply_column_order()
         self.rectree_conf.apply_visibility()
-        self.rectree.connect("row-activated",self.recTreeSelectRec)#self.recTreeSelectRec)
+        self.rectree.connect("row-activated",self.rec_tree_select_rec)#self.rec_tree_select_rec)
         self.rectree.get_selection().connect("changed",self.selection_changedCB)
+        self.rectree.set_property('rules-hint',True) # stripes!
         self.rectree.expand_all()
         self.rectree.show()
 
@@ -201,11 +245,14 @@ class RecIndex:
         """Display the count of currently visible recipes."""
         debug("set_reccount (self, *args):",5)
         self.count = self.rmodel._get_length_()
-        #self.stat.push(self.contid,_("%s Recipes")%self.count)
         bottom,top,total = self.rmodel.showing()
         if top >= total and bottom==1:
             lab = ngettext('%s recipe','%s recipes',top)%top
+            for b in self.first_button,self.prev_button,self.next_button,self.last_button:
+                b.hide()
         else:
+            for b in self.first_button,self.prev_button,self.next_button,self.last_button:
+                b.show()
             # Do not translate bottom, top and total -- I use these fancy formatting
             # strings in case your language needs the order changed!
             lab = _('Showing recipes %(bottom)s to %(top)s of %(total)s'%locals())
@@ -244,7 +291,8 @@ class RecIndex:
                                 'resizable':True},
                     )
                 cssu.set_sort_column_id(twsm.col,twsm.data_col)
-                n += 1                
+                n += 1
+                twsm.col.set_min_width(110)
                 continue
             # And we also special case our time column
             elif c in ['preptime','cooktime']:
@@ -288,7 +336,7 @@ class RecIndex:
                 renderer = gtk.CellRendererCombo()
                 model = gtk.ListStore(str)
                 if c=='category':
-                    map(lambda i: model.append([i]),self.rg.rd.get_unique_values(c,self.rg.rd.catview)
+                    map(lambda i: model.append([i]),self.rg.rd.get_unique_values(c,self.rg.rd.categories_table)
                         )
                 else:
                     map(lambda i: model.append([i]),self.rg.rd.get_unique_values(c))
@@ -296,10 +344,29 @@ class RecIndex:
                 renderer.set_property('text-column',0)
             else:
                 renderer = gtk.CellRendererText()
+                if c=='link':
+                    renderer.set_property('ellipsize',pango.ELLIPSIZE_END)
+                else:
+                    # If we have gtk > 2.8, set up text-wrapping
+                    try:
+                        renderer.get_property('wrap-width')
+                    except TypeError:
+                        pass
+                    else:
+                        renderer.set_property('wrap-mode',gtk.WRAP_WORD)
+                        if c == 'title': renderer.set_property('wrap-width',200)
+                        else: renderer.set_property('wrap-width',150)
             renderer.set_property('editable',self.editable)
             renderer.connect('edited',self.rtree_edited_cb,n, c)
             titl = self.rtcolsdic[c]
             col = gtk.TreeViewColumn('_%s'%titl,renderer, text=n)
+            # Ensure that the columns aren't really narrow on initialising.
+            #if c=='title':            # Adjust these two to be even bigger
+            #    col.set_min_width(200)
+            #else:
+            #    col.set_min_width(60)
+            if c=='title':
+                col.set_property('expand',True)
             col.set_reorderable(True)
             col.set_resizable(True)
             col.set_clickable(True)
@@ -320,10 +387,17 @@ class RecIndex:
 
     def toggleRegexpCB (self, widget):
         """Toggle search-with-regexp option."""
+        #if widget.get_active():
+        #    self.message('Advanced searching (regular expressions) turned on')
+        #else:
+        #    self.message('Advanced searching off')
+        pass
+
+    def toggleShowSearchOptions (self, widget):
         if widget.get_active():
-            self.message('Advanced searching (regular expressions) turned on')
+            self.searchOptionsBox.show()
         else:
-            self.message('Advanced searching off')
+            self.searchOptionsBox.hide()
 
     def regexpp (self):
         """Return True if we're using regexps"""
@@ -357,8 +431,9 @@ class RecIndex:
         elif self.limitButton: self.limitButton.set_sensitive(False)
         if self.make_search_dic(txt,searchBy) == self.last_search:
             debug("Same search!",0)
-            return        
+            return
         if self.srchentry.window: self.srchentry.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
+        debug('Doing new search for %s, last search was %s'%(self.make_search_dic(txt,searchBy),self.last_search),0)
         gobject.idle_add(lambda *args: self.do_search(txt, searchBy))
 
     def make_search_dic (self, txt, searchBy):
@@ -375,7 +450,6 @@ class RecIndex:
     def do_search (self, txt, searchBy):
         if txt and searchBy:
             srch = self.make_search_dic(txt,searchBy)
-            srch['column']=searchBy
             self.last_search = srch.copy()
             self.update_rmodel(self.rd.search_recipes(
                 self.searches + [srch],
@@ -387,17 +461,20 @@ class RecIndex:
                 sort_by=self.sort_by)
                                )
         else:
-            self.update_rmodel(self.rd.fetch_all(self.rview,deleted=False,sort_by=self.sort_by))
+            self.update_rmodel(self.rd.fetch_all(self.recipe_table,deleted=False,sort_by=self.sort_by))
     
     def limit_search (self, *args):
         debug("limit_search (self, *args):",5)
         self.search() # make sure we've done the search...
         self.searches.append(self.last_search)
+        last_col = self.last_search['column']
         self.srchLimitBar.show()
-        if self.srchLimitDefaultText==self.srchLimitText:
-            newtext=_(" %s in %s")%(self.srchentry.get_text(),self.last_search['column'])
+        if last_col != _('anywhere'):
+            newtext = ' ' + _('%s in %s')%(self.srchentry.get_text(),last_col)
         else:
-            newtext=_(", %s in %s")%(self.srchentry.get_text(),self.last_search['column'])
+            newtext = ' ' + self.srchentry.get_text()
+        if self.srchLimitDefaultText!=self.srchLimitLabel.get_text():
+            newtext = ',' + newtext
         self.srchLimitText="%s%s"%(self.srchLimitLabel.get_text(),newtext)
         self.srchLimitLabel.set_markup("<i>%s</i>"%self.srchLimitText)
         self.srchentry.set_text("")
@@ -422,7 +499,7 @@ class RecIndex:
         else:
             secs = self.rg.conv.timestring_to_seconds(text)
             if not secs:
-                self.message(_("Unable to recognize %s as a time."%text))
+                #self.message(_("Unable to recognize %s as a time."%text))
                 return
         indices = path_string.split(':')
         path = tuple( map(int, indices))
@@ -434,7 +511,7 @@ class RecIndex:
             self.rd.undoable_modify_rec(rec,
                                         {attribute:secs},
                                         self.history,
-                                        get_current_rec_method=lambda *args: self.recTreeSelectedRecs()[0],
+                                        get_current_rec_method=lambda *args: self.get_selected_recs_from_rec_tree()[0],
                                         )
             self.update_modified_recipe(rec,attribute,secs)
         # Is this really stupid? I don't know, but I did it before so
@@ -461,7 +538,7 @@ class RecIndex:
             self.rd.undoable_modify_rec(rec,
                                         {attribute:text},
                                         self.history,
-                                        get_current_rec_method=lambda *args: self.recTreeSelectedRecs()[0],
+                                        get_current_rec_method=lambda *args: self.get_selected_recs_from_rec_tree()[0],
                                         )
             self.update_modified_recipe(rec,attribute,text)
         # for metakit, which isn't autocomitting very nicely...        
@@ -478,7 +555,7 @@ class RecIndex:
                 rec,
                 {'rating':value},
                 self.history,
-                get_current_rec_method = lambda *args: self.recTreeSelectedRecs()[0],
+                get_current_rec_method = lambda *args: self.get_selected_recs_from_rec_tree()[0],
                 )
             #self.rmodel.row_changed(self.rmodel.get_path(treeiter),treeiter)
             self.rmodel.update_iter(treeiter)
@@ -490,11 +567,11 @@ class RecIndex:
         the information in the index view."""
         pass
 
-    def recTreeSelectRec (self, *args):
+    def rec_tree_select_rec (self, *args):
         raise NotImplementedError
 
-    def recTreeSelectedRecs (self):
-        debug("recTreeSelectedRecs (self):",5)
+    def get_selected_recs_from_rec_tree (self):
+        debug("get_selected_recs_from_rec_tree (self):",5)
         def foreach(model,path,iter,recs):
             debug("foreach(model,path,iter,recs):",5)
             try:
@@ -533,9 +610,9 @@ class RecIndex:
         except:
             debug('something bizaare just happened in visibility_fun',0)
             return False
- 
-    def update_rmodel (self, rview):
-        self.rmodel.change_view(rview)
+
+    def update_rmodel (self, recipe_table):
+        self.rmodel.change_view(recipe_table)
         self.set_reccount()
 
 class RecipeModel (pageable_store.PageableViewStore):
@@ -598,9 +675,9 @@ class RecipeModel (pageable_store.PageableViewStore):
         for n,row in enumerate(self):
             debug('Looking at row',3)
             if row[0].id==recipe:
-                indx = n + (self.page * self.per_page)
+                indx = int(n + (self.page * self.per_page))
                 # update parent
-                self.parent_list[indx] = self.rd.fetch_one(self.rd.rview,
+                self.parent_list[indx] = self.rd.fetch_one(self.rd.recipe_table,
                                                            id=recipe)
                 # update self
                 self.update_iter(row.iter)
@@ -611,9 +688,9 @@ class RecipeModel (pageable_store.PageableViewStore):
     #    # This is ugly, terrible, no good code. Among other things,
     #    # this is rather specifically metakit hackery which will have
     #    # to be reworked should another backend ever be implemented.
-    #    #sorted_catview = self.rd.catview.sort('category')
+    #    #sorted_categories_table = self.rd.categories_table.sort('category')
     #    #if self.rd.__class__.__module__.find('rmetakit')>=0:
-    #    #    for r in sorted_catview:
+    #    #    for r in sorted_categories_table:
     #    #        if r and r.category:
     #    #            self.rd.modify_rec(r,
     #    #                               {'categoryname':r.category})
