@@ -1,16 +1,17 @@
-#!/usr/bin/env python
+ #!/usr/bin/env python
 import gtk.glade, gtk, gobject, pango, sys, os.path, time, os, string
-import recipeManager, convert, WidgetSaver, reccard
-import dialog_extras as de
-import treeview_extras as te
+import recipeManager, convert, reccard
+from gtk_extras import WidgetSaver, mnemonic_manager
+from gtk_extras import dialog_extras as de
+from gtk_extras import treeview_extras as te
 import exporters.printer as printer
 from gdebug import *
 from gglobals import *
 from gettext import gettext as _
-from nutrition.nutritionLabel import NutritionLabel
-from nutrition.nutrition import NutritionInfoList
-from FauxActionGroups import ActionManager
-import mnemonic_manager
+#from nutrition.nutritionLabel import NutritionLabel
+#from nutrition.nutrition import NutritionInfoList
+from gtk_extras.FauxActionGroups import ActionManager
+
 
 class ShopGui (ActionManager):
     """A class to manage our shopping window."""
@@ -23,7 +24,6 @@ class ShopGui (ActionManager):
         self.mm.fix_conflicts_peacefully()
         self.conv=conv
         self.rg = RecGui
-        self.nd = self.rg.nd # convenient reference to our nutrition
         self.recs = {}
         self.extras = []
         self.data,self.pantry=self.grabIngsFromRecs([])
@@ -60,7 +60,7 @@ class ShopGui (ActionManager):
             'move_to_shopping' : self.rem_selection_from_pantry,
             'move_to_pantry' : self.add_selection_to_pantry,
             'show_help': lambda *args: de.show_faq(HELP_FILE,jump_to='Shopping'),
-            'show_nutritional_info': self.show_nutritional_info,
+            #'show_nutritional_info': self.show_nutritional_info,
             })
 
     def init_action_manager (self):
@@ -115,37 +115,38 @@ class ShopGui (ActionManager):
     def create_rmodel (self):
         debug("create_rmodel (self):",5)
         mod = gtk.TreeStore(gobject.TYPE_PYOBJECT, gobject.TYPE_STRING, gobject.TYPE_STRING)
-        for r,mult in self.recs.items():
+        for r,mult in self.recs.values():
             iter = mod.append(None)
             mod.set_value(iter,0,r)
             mod.set_value(iter,1,r.title)
             mod.set_value(iter,2,convert.float_to_frac(mult))
         return mod
 
-    def show_nutritional_info (self, *args):
-        """Show nutritional information for this shopping list.
-        """
-        nl = NutritionLabel(self.rg.prefs,
-                            custom_label=_('Nutritional Information for Shopping List')
-                            )
-        ings = []
-        for d in [self.sh.dic,self.sh.mypantry]:
-            for k,units in d.items():
-                for a,u in units:
-                    ings.append([k,a,u])
-        nutinfo = NutritionInfoList(
-            [self.nd.get_nutinfo_for_item(*i) for i in ings]
-            )
-        nl.set_nutinfo(nutinfo)
-        sw = gtk.ScrolledWindow()
-        sw.set_policy(gtk.POLICY_NEVER,gtk.POLICY_AUTOMATIC)
-        sw.add_with_viewport(nl)
-        sw.show(); nl.show()
-        md = de.ModalDialog(title=_("Nutritional Information for Shopping List"),modal=False)
-        md.vbox.pack_start(sw,fill=True,expand=True)
-        md.set_default_size(600,500)
-        nl.show()
-        md.run()
+    # def show_nutritional_info (self, *args):
+#         """Show nutritional information for this shopping list.
+#         """
+#         nl = NutritionLabel(self.rg.prefs,
+#                             custom_label=_('Nutritional Information for Shopping List')
+#                             )
+#         ings = []
+#         for d in [self.sh.dic,self.sh.mypantry]:
+#             for k,units in d.items():
+#                 for a,u in units:
+#                     ings.append([k,a,u])
+#         nutinfo = NutritionInfoList(
+#             [self.nd.get_nutinfo_for_item(*i) for i in ings]
+#             )
+#         nl.set_nutinfo(nutinfo)
+#         sw = gtk.ScrolledWindow()
+#         sw.set_policy(gtk.POLICY_NEVER,gtk.POLICY_AUTOMATIC)
+#         sw.add_with_viewport(nl)
+#         sw.show(); nl.show()
+#         md = de.ModalDialog(title=_("Nutritional Information for Shopping List"),modal=False)
+#         md.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_NORMAL)
+#         md.vbox.pack_start(sw,fill=True,expand=True)
+#         md.set_default_size(600,500)
+#         nl.show()
+#         md.show()
 
     def save (self, *args):
         debug("save (self, *args):",5)
@@ -177,7 +178,7 @@ class ShopGui (ActionManager):
         w = printer(*args,**kwargs)
         w.write_header(_("Shopping list for %s")%time.strftime("%x"))
         w.write_subheader(_("For the following recipes:"))
-        for r,mult in self.recs.items():
+        for r,mult in self.recs.values():
             itm = "%s"%r.title
             if mult != 1:
                 itm += _(" x%s")%mult
@@ -205,7 +206,7 @@ class ShopGui (ActionManager):
         selectedRecs=self.getSelectedRecs()
         if selectedRecs:
             for t in selectedRecs:
-                self.recs.__delitem__(t)
+                self.recs.__delitem__(t.id)
                 debug("clear removed %s"%t,3)
             self.resetSL()
         elif de.getBoolean(label=_("No recipes selected. Do you want to clear the entire list?")):
@@ -219,7 +220,7 @@ class ShopGui (ActionManager):
         debug("addRec (self, rec, mult, includes={}):",5)
         """Add recipe to our list, assuming it's not already there.
         includes is a dictionary of optional items we want to include/exclude."""
-        self.recs[rec]=mult
+        self.recs[rec.id]=(rec,mult)
         self.includes[rec.id]=includes
         self.resetSL()
 
@@ -247,7 +248,7 @@ class ShopGui (ActionManager):
         
     def resetSL (self):
         debug("resetSL (self):",5)
-        self.data,self.pantry = self.grabIngsFromRecs(self.recs.items(),self.extras)
+        self.data,self.pantry = self.grabIngsFromRecs(self.recs.values(),self.extras)
         self.slMod = self.createIngModel(self.data)
         self.pMod = self.createIngModel(self.pantry)
         self.slTree.set_model(self.slMod)
@@ -457,7 +458,7 @@ class ShopGui (ActionManager):
         sublab = ', '.join(kk)
         cat = de.getEntry(label=_('Enter Category'),
                           sublabel=_("Category to add %s to") %sublab,
-                          entryLabel=_('Category: '),
+                          entryLabel=_('Category:'),
                           parent=self.widget)
         if cat:
             for k in kk:
@@ -574,12 +575,9 @@ class ShopGui (ActionManager):
         """Handed an array of (rec . mult)s, we combine their ingredients.
         recs may be IDs or objects."""
         lst = start[0:]
-        for r in recs:
-            if len(r) > 1:
-                lst.extend(self.grabIngFromRec(r[0],mult=r[1]))
-            else:
-                lst.extend(self.grabIngFromRec(r[0]))
-        self.sh = recipeManager.mkShopper(lst, self.rg.rd, conv=self.conv)
+        for rec,mult in recs:
+            lst.extend(self.grabIngFromRec(rec,mult=mult))
+        self.sh = recipeManager.DatabaseShopper(lst, self.rg.rd, conv=self.conv)
         data = self.sh.organize(self.sh.dic)
         pantry = self.sh.organize(self.sh.mypantry)
         debug("returning: data=%s pantry=%s"%(data,pantry),5)
@@ -587,15 +585,11 @@ class ShopGui (ActionManager):
             
     def grabIngFromRec (self, rec, mult=1):
         """Get an ingredient from a recipe and return a list with our amt,unit,key"""
-        debug("grabIngFromRec (self, rec=%s(%s), mult=%s):"%(rec.title,rec.id,mult),5)
         """We will need [[amt,un,key],[amt,un,key]]"""
-        if type(rec) == type(""): #if we have an ID (type str), grab the recipe from it
-            rec = self.rg.rd.get_rec(rec)
-        ings = self.rg.rd.get_ings(rec) # grab all of our ingredients
+        debug("grabIngFromRec (self, rec=%s, mult=%s):"%(rec,mult),5)
+        # Grab all of our ingredients
+        ings = self.rg.rd.get_ings(rec)
         lst = []
-        # this constant should be 'ask',or rdatabase.RecData.AMT_MODE_LOW|AMT_MODE_AVERAGE|AMT_MODE_HIGH
-        # this is handled in preferences (see prefsGui.py)
-        #RESOLVE_RANGE_METHOD=self.rg.prefs.get('shop_handle_ranges','ask')
         include_dic = self.includes.get(rec.id) or {}
         for i in ings:
             if hasattr(i,'refid'): refid=i.refid
@@ -603,12 +597,15 @@ class ShopGui (ActionManager):
             debug("adding ing %s, %s"%(i.item,refid),4)
             if i.optional:
                 # handle boolean includes value which applies to ALL ingredients
-                if not include_dic or (include_dic != True and 
-                                       (not include_dic.has_key(i.ingkey)
-                                        or not include_dic[i.ingkey])
-                                       ):
-                    # we ignore our ingredient (don't add it)
+                if not include_dic:
                     continue
+                if type(include_dic) == dict :
+                    # Then we have to look at the dictionary itself...
+                    if ((not include_dic.has_key(i.ingkey))
+                        or
+                        not include_dic[i.ingkey]):
+                        # we ignore our ingredient (don't add it)
+                        continue
             if self.rg.rd.get_amount(i):
                 amount=self.rg.rd.get_amount(i,mult=mult)                
             else: amount=None            
@@ -671,7 +668,7 @@ class ShopGui (ActionManager):
 
     def setup_shop_dialog (self):
         self.orgdic = self.sh.orgdic
-        self.sie = shopIngredientEditor(self.rg, self)
+        self.sie = ShopIngredientEditor(self.rg, self)
 
     def message (self, txt):
         debug("message (self, txt): %s"%txt,5)
@@ -762,15 +759,16 @@ class OptionalIngDialog (de.ModalDialog):
             for row in self.mod:
                 ing = row[0]
                 ing_include = row[4]
-                if ing_include: ing.shopoptional=2
-                else: ing.shopoptional=1
+                if ing_include: self.rg.rd.modify_ing(ing,{'shopoptional':2})
+                else: self.rg.rd.modify_ing(ing,{'shopoptional':1})
         return self.ret
 
-class shopIngredientEditor (reccard.IngredientEditor):
+class ShopIngredientEditor (reccard.IngredientEditor):
     def __init__ (self, RecGui, ShopGui):
         self.sg = ShopGui
         self.last_key = ""
         reccard.IngredientEditor.__init__(self, RecGui, None)
+        #self.ingBox = self.keyBox
 
     def init_dics (self):
         self.orgdic = self.sg.sh.orgdic
@@ -824,6 +822,10 @@ class shopIngredientEditor (reccard.IngredientEditor):
             self.sg.extras.append(itm)
             self.sg.resetSL()
             self.new()
+
+    def returned (self, *args):
+        self.add()
+        self.amountBox.grab_focus()
         
     def toggleShopDialog (self, *args):
         if self.sdToggle: self.showShopDialog()
